from buildings import *
from time import time
from random import *
INFO = """
Population: %(population)i

Credits:$%(money).2f
    Taxes:$%(total_tax).2f
    Commerce:$%(total_commerce).2f
    Upkeep:$%(total_upkeep).2f
    Unemployment:%(pay_unemployment).2f %(unemployment)i %%
    
    Power IN:%(make_power).1f OUT:%(take_power).1f
    Water IN:%(make_water)i OUT:%(take_water)i
    """

def format(format,dict):
    import re
    keys=[]
    def find_braces(a):
        keys.append(a.group(1))
        return ""
    re.sub(r"\((.*?)\)", find_braces , format)
    d = {}
    for key in keys:
        d[key] = dict[key]
    return format%d

class RevenueSystem:
    
    def __init__(self):
        self.money = 10000
        self.min_mount = 0
        self.unemployment = 0
    
        self.population = 0
        self.total_tax = 0
        self.total_upkeep = 0
        self.total_commerce = 0
        self.make_power = 0
        self.take_power = 0
        self.make_water = 0
        self.take_water = 0
        self.pay_unemployment = 0
        self.unemployment = 0
    
    def houses(self,populations):
        self.population += populations
        
    def check(self,amount):
        if self.money - self.min_mount > amount:
            return True
    
    def power(self,amount):
        if amount > 0:
            self.make_power += amount
        elif amount < 0:
            self.take_power -= amount
            
    def water(self,amount):
        if amount > 0:
            self.make_water += amount
        elif amount < 0:
            self.take_water -= amount
            
    def withdraw(self,amount):
        self.money -= amount
        
    def deposit(self,amoutn):
        self.money += amount
    
    def commurce(self,amount):
        self.total_commerce += amount
        self.money += amount
        
    def upkeep(self,amount):
        self.total_upkeep += amount
        self.money -= amount
    
    def update(self):
        output = format(INFO,self.__dict__)
        self.total_tax = 0
        self.total_upkeep = 0
        self.total_commerce = 0   
        self.make_power = 0
        self.take_power = 0
        self.make_water = 0
        self.take_water = 0  
        self.population = 0   
        return output
    

class BuildingSystem:
    
    buildings = []
        
    def over(self,building,position):
        if self.over_closest(building,position):
            game.ms.post(str(self.closest)+" is in the way")
            return True
        return False
        
    def over_closest(self,building,position,radius=None):
        r = building.radius
        for b in self.buildings:
            d = (b.node.getPos()-position).length()
            if not radius :
                if d < r+b.radius:
                    self.closest = b
                    return True
            else:
                if d < radius:
                    self.closest = b
                    return True
        return False
    
    def remove(self,p):
        dist = 9999
        to_remove = None
        for b in self.buildings:
            d = (b.node.getPos()-p).length()
            if d < dist:
               to_remove = b
               dist = d 
        if to_remove:
            to_remove.node.removeNode()
            self.buildings.remove(to_remove)
            if to_remove.system:
                for node in to_remove.connected:
                    node.connected.remove(to_remove)
                to_remove.system.remove(to_remove)
                game.gen_lines()
            
                
    
    def place(self,position,type):
        if not game.rv.check(type.cost):
            game.ms.post("not enough cash")
            return
        game.rv.withdraw(type.cost)  

        if (abs(game.normal[0]) + abs(game.normal[1])) > game.normal[2]:
            game.ms.post("too steep to build here")
            return False
        
        building = type()
        if not self.over(building,position) and building.can_build_check(position):
            self.buildings.append(building)
            return building.load()
           
        #RoadNode
    def lay(self,start,end,type,system):
        
        if not game.rv.check(type.cost):
            game.ms.post("not enough cash")
            return 
        game.rv.withdraw(type.cost)   
        
        start_roadNode = end_roadNode = None
        if (start-end).length() < 1:
            game.ms.post("segment too short")
        elif (start-end).length() > 15:
            game.ms.post("segment too long")
        else:
            ok = False
            start_node = type()
            if not self.over_closest(start_node,start,1):
                ok = True
            elif self.closest.__class__ == type:
                ok = True
                start = self.closest.node.getPos()
                start_roadNode = self.closest
                start_node = False

            if ok:
                end_node = type()   
                if not self.over_closest(end_node,end,1):
                    ok = True
                elif self.closest.__class__ == type:
                    ok = True
                    end = self.closest.node.getPos()
                    end_roadNode = self.closest
                    end_node = False
                else:
                    ok = False
               
            if ok :
                
                if start_roadNode and end_roadNode:
                    if system.connected(start_roadNode,end_roadNode):
                        game.ms.post("there already is a point between 2 nodes")
                        return False
                    
                if start_node : 
                    start_roadNode = start_node
                    self.buildings.append(start_node)
                    building = start_node.load()
                    building.setPos(start)  
                    building.lookAt(Point3(end))
                    building.setR(0)
                    building.setP(0)
                
                if end_node : 
                    end_roadNode = end_node
                    self.buildings.append(end_node)
                    building = end_node.load()
                    building.setPos(end)  
                    building.lookAt(Point3(start))
                    building.setR(0)
                    building.setP(0)
                        
                system.connect(start_roadNode,end_roadNode)
                    
                return end_roadNode
                
         
    def simulate(self):
        self.buildings.sort(key=lambda b: b.order)
        for building in self.buildings:
            building.simulate()
        for building in self.buildings:
            building.simulate()
            building.after_simulate()
            
class RoadSystem:
    
    node_name = 'transport hub'
    
    def __init__(self):
        self.roads = []
    
    def faceTo(self,pos):
        f = Point3(50,50,0)
        dist,closest = self.find_closest_road(pos)
        if closest: f = closest.node.getPos()
        f.setZ(0)
        return Point3(f)
    
    def check(self,building,type,amount):        
        dist,closest = self.find_closest_road(building.node.getPos())
        return dist < 10 and closest.check(type) >= amount  
    
    def get(self,building,type,amount):        
        dist,closest = self.find_closest_road(building.node.getPos())
        if dist < 10 :
            return closest.check(type)
        else: 
            return 0
    
    def find_closest_road(self,pos):
        dist = 99999
        closest = None
        for road in self.roads:
            d = (road.node.getPos()-pos).length()  
            if d < dist:
                dist = d
                closest = road
        return dist,closest
    
    def drain(self,building,type,amount):
        dist,closest = self.find_closest_road(building.node.getPos())
        if dist < 10:
            closest.drain(type,amount)  
    
    def pour(self,building,type,amount):
        dist,closest = self.find_closest_road(building.node.getPos())
        if dist < 10:
            closest.pour(type,amount)  
        else:
            game.ms.post('%s is not close to any %s (%i) '%(str(building),self.node_name,int(dist)),-200)      
        
        #find closest road node and add it
        
    def connected(self,a,b):
            
        if a in b.connected:
            return True
        if b in a.connected:
            game.ms.post(self.node_name+" integrity error! *** what have you done? ")
            return True
        return False
    
    def remove(self,node):
        self.roads.remove(node)
    
    def connect(self,a,b):
        if a not in self.roads: self.roads.append(a)
        if b not in self.roads: self.roads.append(b)
        b.connected.append(a)
        b.system = self
        a.connected.append(b)
        a.system = self
        
    def gen_lines(self,lines):
        for road in self.roads:
            for c in road.connected: 
                lines.moveTo(road.node.getPos()+ Vec3(0,0,.3))
                lines.drawTo(c.node.getPos()+ Vec3(0,0,.3))
        
class WaterSystem(RoadSystem):
    
    node_name = 'aqua node'
        
class PowerSystem(RoadSystem):
    
    node_name = 'energy node'
  
    def too_close(self,this_building,p):
        if this_building.safe_zone:
            # find building with no save zone
            for b in game.bs.buildings:
                if not b.safe_zone and not b.system:
                    if (b.node.getPos()-p).length() < this_building.safe_zone:
                        game.ms.post(str(this_building)+" will have nuclear reactor, can't be build close to a "+str(b))
                        return True
        if not this_building.safe_zone:
            # find building with no save zone
            for b in game.bs.buildings:
                if b.safe_zone:
                    if (b.node.getPos()-p).length() < b.safe_zone:
                        game.ms.post(str(this_building)+" can't build so close to nuclear reactor in "+str(b) )
                        return True            
        return False
        
class DemandSystem:
    
    outside = 0
    inside = 1
    
    def simulate(self):
        #randomly change goods around
        pass
        
               
class MessageSystem:
    
    messages = []
    i = 0
    
    def update(self):
        now = time()
        for h in list(self.messages):
            if now - h[1] > 2:
                self.messages.remove(h)
        self.messages.sort(key=lambda h:-h[2])
        game.out['text'] = "\n".join([ h[0] for h in self.messages])
           
    def post(self,message,priority=0):
        
        if "Command is not close to any" in message: return
        
        print message,priority
        # make shure they dont shift 
        priority += random() 
        now = time()
        # check to see if message is there:
        for h in self.messages:
            if message == h[0]:
                h[1] = now
                message = None
        if message:
            self.messages.append([message,now,priority])
        self.update()
       